"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const core_1 = require("@salesforce/core");
const fast_xml_parser_1 = require("fast-xml-parser");
const componentSet_1 = require("./componentSet");
const client_1 = require("../client");
const common_1 = require("../common");
const errors_1 = require("../errors");
const i18n_1 = require("../i18n");
const metadata_registry_1 = require("../metadata-registry");
class WorkingSet {
    constructor(registry = new metadata_registry_1.RegistryAccess()) {
        this.components = new Map();
        this.registry = registry;
        this.apiVersion = registry.apiVersion;
    }
    /**
     * Create a set by resolving components from source.
     *
     * @param fsPath Path to resolve components from
     * @param options
     */
    static fromSource(fsPath, options) {
        var _a, _b;
        const ws = new WorkingSet((_a = options) === null || _a === void 0 ? void 0 : _a.registry);
        ws.resolveSourceComponents(fsPath, { tree: (_b = options) === null || _b === void 0 ? void 0 : _b.tree });
        return ws;
    }
    /**
     * Create a set by reading a manifest file in xml format. Optionally, specify a file path
     * with the `resolve` option to resolve source files for the components.
     *
     * ```
     * WorkingSet.fromManifest('/path/to/package.xml', {
     *  resolve: '/path/to/force-app'
     * });
     * ```
     *
     * @param fsPath Path to xml file
     * @param options
     */
    static fromManifestFile(fsPath, options) {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            const registry = (_b = (_a = options) === null || _a === void 0 ? void 0 : _a.registry, (_b !== null && _b !== void 0 ? _b : new metadata_registry_1.RegistryAccess()));
            const tree = (_d = (_c = options) === null || _c === void 0 ? void 0 : _c.tree, (_d !== null && _d !== void 0 ? _d : new metadata_registry_1.NodeFSTreeContainer()));
            const shouldResolve = (_e = options) === null || _e === void 0 ? void 0 : _e.resolve;
            const ws = new WorkingSet((_f = options) === null || _f === void 0 ? void 0 : _f.registry);
            const filterSet = new componentSet_1.ComponentSet();
            const file = yield tree.readFile(fsPath);
            const manifestObj = fast_xml_parser_1.parse(file.toString(), {
                stopNodes: ['version'],
            });
            ws.apiVersion = manifestObj.Package.version;
            for (const component of WorkingSet.getComponentsFromManifestObject(manifestObj, registry)) {
                const memberIsWildcard = component.fullName === WorkingSet.WILDCARD;
                if (shouldResolve) {
                    filterSet.add(component);
                }
                if (!shouldResolve || (memberIsWildcard && ((_g = options) === null || _g === void 0 ? void 0 : _g.literalWildcard))) {
                    ws.add(component);
                }
            }
            if (shouldResolve) {
                ws.resolveSourceComponents(options.resolve, {
                    tree,
                    filter: filterSet,
                });
            }
            return ws;
        });
    }
    /**
     * Create a set from a collection of components or type members.
     *
     * @param collection
     * @param options
     */
    static fromComponents(collection, options) {
        var _a;
        const ws = new WorkingSet((_a = options) === null || _a === void 0 ? void 0 : _a.registry);
        for (const component of collection) {
            ws.add(component);
        }
        return ws;
    }
    static *getComponentsFromManifestObject(obj, registry) {
        const { types } = obj.Package;
        const typeMembers = Array.isArray(types) ? types : [types];
        for (const { name: typeName, members } of typeMembers) {
            const fullNames = Array.isArray(members) ? members : [members];
            for (const fullName of fullNames) {
                yield {
                    fullName,
                    type: registry.getTypeByName(typeName),
                };
            }
        }
    }
    /**
     * Deploy components in the set to an org. The components must be backed by source files.
     * Deploying with a username requires local AuthInfo from @salesforce/core, usually created
     * after authenticating with the Salesforce CLI.
     *
     * @param usernameOrConnection Username or connection to deploy components with.
     * @param options
     */
    deploy(usernameOrConnection, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const toDeploy = [];
            const missingSource = [];
            for (const component of this) {
                if (component instanceof metadata_registry_1.SourceComponent) {
                    toDeploy.push(component);
                }
                else {
                    if (component.fullName === WorkingSet.WILDCARD) {
                        throw new errors_1.WorkingSetError('error_deploy_wildcard_literal');
                    }
                    missingSource.push(`${component.type.name}:${component.fullName}`);
                }
            }
            if (toDeploy.length === 0) {
                throw new errors_1.WorkingSetError('error_no_source_to_deploy');
            }
            else if (missingSource.length > 0) {
                console.warn(i18n_1.nls.localize('warn_unresolved_source_for_components', missingSource.join(',')));
            }
            const connection = yield this.getConnection(usernameOrConnection);
            const client = new client_1.SourceClient(connection, new metadata_registry_1.MetadataResolver());
            return client.metadata.deploy(toDeploy, options);
        });
    }
    /**
     * Retrieve components in the set from an org. Components are not required to be backed by
     * source files. Retrieving with a username requires local AuthInfo from @salesforce/core,
     * usually created after authenticating with the Salesforce CLI.
     *
     * @param usernameOrConnection Username or Connection to retrieve with.
     * @param output Directory to retrieve to.
     * @param options
     */
    retrieve(usernameOrConnection, output, options) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const connection = yield this.getConnection(usernameOrConnection);
            const client = new client_1.SourceClient(connection, new metadata_registry_1.MetadataResolver());
            if (this.components.size === 0) {
                throw new errors_1.WorkingSetError('error_no_components_to_retrieve');
            }
            return client.metadata.retrieve({
                // this is fine, if they aren't mergable then they'll go to the default
                components: Array.from(this),
                merge: (_a = options) === null || _a === void 0 ? void 0 : _a.merge,
                output,
                wait: (_b = options) === null || _b === void 0 ? void 0 : _b.wait,
            });
        });
    }
    /**
     * Get an object representation of a package manifest based on the set components.
     */
    getObject() {
        const typeMembers = [];
        for (const [typeName, components] of this.components.entries()) {
            const members = [];
            for (const { fullName } of components.values()) {
                members.push(fullName);
            }
            typeMembers.push({ members, name: typeName });
        }
        return {
            Package: {
                types: typeMembers,
                version: this.apiVersion,
            },
        };
    }
    /**
     * Resolve source backed components and add them to the set.
     *
     * @param fsPath: File path to resolve
     * @param options
     */
    resolveSourceComponents(fsPath, options) {
        var _a, _b, _c;
        let filterSet;
        if ((_a = options) === null || _a === void 0 ? void 0 : _a.filter) {
            const { filter } = options;
            filterSet = filter instanceof componentSet_1.ComponentSet ? filter : new componentSet_1.ComponentSet(filter);
        }
        // TODO: move filter logic to resolver W-8023153
        const resolver = new metadata_registry_1.MetadataResolver(this.registry, (_b = options) === null || _b === void 0 ? void 0 : _b.tree);
        const resolved = resolver.getComponentsFromPath(fsPath);
        const sourceComponents = new componentSet_1.ComponentSet();
        for (const component of resolved) {
            const shouldResolve = !filterSet || filterSet.has(component);
            const includedInWildcard = (_c = filterSet) === null || _c === void 0 ? void 0 : _c.has({
                fullName: WorkingSet.WILDCARD,
                type: component.type,
            });
            if (shouldResolve || includedInWildcard) {
                this.setComponent(component);
                sourceComponents.add(component);
            }
            else if (filterSet) {
                for (const childComponent of component.getChildren()) {
                    if (filterSet.has(childComponent)) {
                        this.setComponent(childComponent);
                        sourceComponents.add(childComponent);
                    }
                }
            }
        }
        return sourceComponents;
    }
    /**
     * Create a manifest in xml format (package.xml) based on the set components.
     *
     * @param indentation Number of spaces to indent lines by.
     */
    getPackageXml(indentation = 4) {
        const j2x = new fast_xml_parser_1.j2xParser({
            format: true,
            indentBy: new Array(indentation + 1).join(' '),
            ignoreAttributes: false,
        });
        const toParse = this.getObject();
        toParse.Package[common_1.XML_NS_KEY] = common_1.XML_NS_URL;
        return common_1.XML_DECL.concat(j2x.parse(toParse));
    }
    add(component) {
        if (typeof component.type === 'string') {
            this.setComponent({
                fullName: component.fullName,
                type: this.registry.getTypeByName(component.type),
            });
        }
        else {
            this.setComponent(component);
        }
    }
    has(component) {
        var _a;
        const typeName = typeof component.type === 'string'
            ? this.registry.getTypeByName(component.type).name
            : component.type.name;
        return ((_a = this.components.get(typeName)) === null || _a === void 0 ? void 0 : _a.has(component)) === true;
    }
    /**
     * Get the set entries grouped by metadata type name.
     *
     * entry -> [type name, component set]
     */
    entries() {
        return this.components.entries();
    }
    *[Symbol.iterator]() {
        for (const componentSet of this.components.values()) {
            for (const component of componentSet) {
                yield component;
            }
        }
    }
    get size() {
        let count = 0;
        for (const set of this.components.values()) {
            count += set.size;
        }
        return count;
    }
    getConnection(auth) {
        return __awaiter(this, void 0, void 0, function* () {
            return auth instanceof core_1.Connection
                ? auth
                : yield core_1.Connection.create({
                    authInfo: yield core_1.AuthInfo.create({ username: auth }),
                });
        });
    }
    setComponent(component) {
        const { type } = component;
        if (!this.components.has(type.name)) {
            this.components.set(type.name, new componentSet_1.ComponentSet());
        }
        this.components.get(type.name).add(Object.freeze(component));
    }
}
exports.WorkingSet = WorkingSet;
WorkingSet.WILDCARD = '*';
//# sourceMappingURL=workingSet.js.map